package pogr;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 *
 * @author KarelSimanek
 */
public class Ascii {
    
    /**
     * Method for asciify the input image into the map of symbols in the same color depth.
     * @param im input image
     * @param f font size
     * @return BufferedImage of symbols of font f
     *
     */
    public BufferedImage Asciify(BufferedImage im, Font f)
    {
        BufferedImage newImage = new BufferedImage(im.getWidth(), im.getHeight(),im.getType());
        Graphics2D g = newImage.createGraphics();
        g.setFont(f);
        FontMetrics metrics = g.getFontMetrics();
        int fontHeight = metrics.getAscent();
        int fontWidth = metrics.getWidths()[0];

        for (int y = 0; y < im.getHeight(); y = y + fontHeight)
        {
            for (int x = 0; x < im.getWidth(); x = x + fontWidth)
            {
                int red = 0;
                int green = 0;
                int blue = 0;
                int brightness = 0; 
                Color pixelColor;
                
                for (int i = 0 ; i < fontHeight; i++) {
                    for (int j = 0; j < fontWidth; j++)
                    {
                        pixelColor = new Color(im.getRGB(x,y));  
                        red += pixelColor.getRed();
                        blue += pixelColor.getBlue();
                        green += pixelColor.getGreen();
                        brightness += pixelColor.getGreen()*0.1140+pixelColor.getBlue()*0.5870+pixelColor.getRed()*0.2989;
                    }
                }
                
                g.setColor(new Color(red/(fontHeight*fontWidth),green/(fontHeight*fontWidth),blue/(fontHeight*fontWidth)));
                g.drawString(getChar(g,brightness/(fontHeight*fontWidth)), x, y);
                
            }
        }
        return newImage;
    }
    
    /**
     * Auxiliary method for converting the mask of the bitmap into the output character
     * @param g Graphics object of the bitmap
     * @param value Font size
     * @return Ascii value of the output char
     */
    private String getChar(Graphics g, double value)
    {
        Map m = LookUpTable(g);
        Iterator i = m.entrySet().iterator();
        Map.Entry me = null;

        while (i.hasNext()){
            me = (Map.Entry)i.next();
            if (((int)me.getKey()) > value) {
                break;
            }
        }

        byte[] bytes = new byte[1];
        bytes[0] = (byte)((int)(me.getValue()));

        return new String(bytes);     	
    }
    
    /**
     * Auxiliary method for keeping the lookUpTable of character intensity
     * @param g Graphics object of the bitmap
     * @return Map of Characters asociated to their intensity
     * 
     */
    private Map LookUpTable(Graphics g)
    {
        FontMetrics metrics = g.getFontMetrics();
        int fontHeight = metrics.getAscent();
        int fontWidth = metrics.getWidths()[0];

        BufferedImage im  = new BufferedImage(fontWidth*95,fontHeight,BufferedImage.TYPE_INT_RGB);
        g = (Graphics2D) im.getGraphics();
        g.getFont();
        
        int actualWidth = 0;
        for (int j = 32; j < 128; j++) {
            byte[] bytes = new byte[1];
            bytes[0] = (byte)(j);
            String letter = new String(bytes);
            g.drawString(letter,actualWidth,fontHeight);
            actualWidth+=fontWidth;
        }
       
        Map m = new TreeMap();
        Color pixelColor;
        
        for (int i=0; i < 95; i++)
        {
        	int brightness = 0;
        	for (int x = 0; x < fontWidth; x++)
        	{
        		for (int y = 0; y < fontHeight; y++)
        		{
                            pixelColor = new Color(im.getRGB(x+i*fontWidth,y));  
                            brightness += pixelColor.getGreen()*0.1140+pixelColor.getBlue()*0.5870+pixelColor.getRed()*0.2989;
        		}
        	}
        	m.put(brightness/(fontWidth*fontHeight),i+32);
        }
        
        //scalovani
        Iterator i = m.entrySet().iterator();
        Map.Entry me = (Map.Entry)i.next();
        //minimum
        int min = (int)me.getKey();
        //maximum
        while (i.hasNext()) {
            me = (Map.Entry)i.next();
        }
        int max = (int)me.getKey();
        
        Iterator j = m.entrySet().iterator();
        Map scaledMap = new TreeMap();
        while (j.hasNext()){
        	me = (Map.Entry)j.next();
        	scaledMap.put(255*((int)me.getKey()-min)/(max-min), me.getValue());
        }
        
        return scaledMap;
    }
}
