package pogr;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
/**
 * Class representing the dialog frame for selecting desired dithering method
 * @author KarelSimanek
 */
public class DitherSelector extends JDialog{
    
    /**
     * Private variable representing the chosen method
     */
    private static String method;
    /**
     * Private variable representing the successful confirmation of the dialog
     */
    private static boolean OK;
    
    /**
     * Auxiliary method for setting the confirmation to OK
     * @param value boolean operand
     */
    private void setOK(boolean value)
    {
        DitherSelector.OK = value;
    }
    
    /**
     * Auxiliarz method for determining the OK value
     * @return boolean value of variable OK
     */
    public boolean isOK()
    {
        return DitherSelector.OK;
    }
    
    /**
     * Constructor for the custom dialog
     * @param parent inherited constructor operand
     */
    public DitherSelector(JFrame parent)
    {
        super(parent,"Dithering method selector");
        
        this.setModal(true);
        this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        
        addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
            public void windowClosing(java.awt.event.WindowEvent evt) {
                setOK(false);
                setVisible(false);
            }
        });
        
        String[] items = {"Floyd-Steinberg","Atkinson","Jarvice-Judice-Ninke","Stucki","Bayer 2x2","Bayer 4x4","Bayer 8x8","Clustered 4x4","Random"};
        selectBox = new JComboBox(items);
        selectBox.setBounds(10,10,150,20);
        selectBox.addActionListener(new ActionListener()
        {
            @Override 
            public void actionPerformed(ActionEvent e)
            {
                SetValues();
                panel.repaint();
            }
        });
        SetValues();
        
        confirmButton = new JButton();
        confirmButton.setBounds(10,200,120,30);
        confirmButton.setText("Process");
        panel = new JPanel()
        {
            @Override
            public void paintComponent(Graphics g)
            {
                BufferedImage sample = null;
                g.setColor(Color.WHITE);
                g.fillRect(0,0, 200, 200);
                BufferedImage originalImage = Window.canvas.getOriginalSizeImage();
                Image scaledImage;
                
                scaledImage = originalImage.getScaledInstance(140,140,Image.SCALE_SMOOTH);
                                
                if (method.equals("Floyd-Steinberg")) {
                    sample = Dither.FloydSteinberg(imageToBufferedImage(scaledImage));
                }
                if (method.equals("Atkinson")) {
                    sample = Dither.Atkinson(imageToBufferedImage(scaledImage));
                }
                if (method.equals("Jarvice-Judice-Ninke")) {
                    sample = Dither.JarvisJudiceNinke(imageToBufferedImage(scaledImage));
                }
                if (method.equals("Stucki")) {
                    sample = Dither.Stucki(imageToBufferedImage(scaledImage));
                }
                if (method.equals("Bayer 2x2")) {
                    sample = Dither.Bayer_2x2(imageToBufferedImage(scaledImage));
                }
                if (method.equals("Bayer 4x4")) {
                    sample = Dither.Bayer_4x4(imageToBufferedImage(scaledImage));
                }
                if (method.equals("Bayer 8x8")) {
                    sample = Dither.Bayer_8x8(imageToBufferedImage(scaledImage));
                }
                if (method.equals("Clustered 4x4")) {
                    sample = Dither.Clustered_4x4(imageToBufferedImage(scaledImage));
                }
                if (method.equals("Random")) {
                    sample = Dither.Random(imageToBufferedImage(scaledImage));
                }
                g.drawImage(sample, 5, 5, null);
            }
        };
       
        panel.setBounds(10,40,150,150);
        
        this.add(panel);
        
        this.setSize(200,300);
        this.setLayout(null);
        this.add(confirmButton);
        this.add(panel);
        this.add(selectBox);
        
        confirmButton.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                SetValues();
                setOK(true);
                setVisible(false);
            }
        });
    }
     /**
     * Auxiliary method for converting format Image to BufferedImage
     * @param im Input image
     * @return BufferedImage converted from the image
     */
    public static BufferedImage imageToBufferedImage(Image im) {
        BufferedImage bi = new BufferedImage
           (im.getWidth(null),im.getHeight(null),BufferedImage.TYPE_INT_RGB);
        Graphics bg = bi.getGraphics();
        bg.drawImage(im, 0, 0, null);
        bg.dispose();
        return bi;
    }
    
    /**
     * Auxiliary method for setting the private variables from the selectBox
     */
    private void SetValues()
    {
        DitherSelector.method = selectBox.getSelectedItem().toString();
    }
    
    /**
     * Auxiliarz method for getting the private variables from the selector
     * @return 
     */
    public String getValues()
    {
        return DitherSelector.method;
    }
    
    /**
     * Displayed ComboBox for selecting the desired method.
     */
    private JComboBox selectBox;
    /**
     * Displayed confirm button for confirmation and closing the window.
     */
    private JButton confirmButton;
    /**
     * Panel for displaying the miniature of the procedure applied on the original image
     */
    private JPanel panel;
    
}
