package pogr;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Class representing the dialog frame for selecting the properties for making the grayscale image
 * @author KarelSimanek
 */
public class GrayScaleSelector extends JDialog{
    
    /**
     * private value of red slider
     */
    private int redValue;
    /**
     * private value of green slider
     */
    private int greenValue;
    /**
     * private value of blue slider
     */
    private int blueValue;
     /**
     * Private variable representing the successful confirmation of the dialog
     */
    private static boolean OK;
    
    /**
     * Auxiliary method for setting the confirmation to OK
     * @param value boolean operand
     */
    private void setOK(boolean value)
    {
        GrayScaleSelector.OK = value;
    }
    /**
     * Auxiliarz method for determining the OK value
     * @return boolean value of variable OK
     */
    public boolean isOK()
    {
        return GrayScaleSelector.OK;
    }
     /**
     * Constructor for the custom dialog
     * @param parent inherited constructor operand
     */
    public GrayScaleSelector(JFrame parent)
    {
        super(parent,"GrayScale Selector");
        
        this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        this.setModal(true);
        
        this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        
        addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
            public void windowClosing(java.awt.event.WindowEvent evt) {
                setOK(false);
                setVisible(false);
            }
        });
        
        redLabel = new JLabel();
        redLabel.setBounds(10,20,110,20);
        redLabel.setText("(%) of  Red:");
        redLabel.setFont(new Font(Font.SERIF,Font.BOLD,15));
        
        redSlider = new JSlider(0,100);
        redSlider.setBounds(145,10,200,50);
        redSlider.setMinorTickSpacing(1);
        redSlider.setMajorTickSpacing(50);
        redSlider.setPaintLabels(true);
        redSlider.addChangeListener(new ChangeListener()
        {
            @Override 
            public void stateChanged(ChangeEvent e)
            {
                redTextValue.setText(""+redSlider.getValue());
                panel.repaint();
            }
        });
        
        redTextValue = new JTextField();
        redTextValue.setBounds(110,20,30,20);    
        redTextValue.setEnabled(false);
        redTextValue.setText(""+redSlider.getValue());
        
        greenLabel = new JLabel();
        greenLabel.setBounds(10,80,110,20);
        greenLabel.setText("(%) of  Green:");
        greenLabel.setFont(new Font(Font.SERIF,Font.BOLD,15));
        
        greenSlider = new JSlider();
        greenSlider.setBounds(145,70,200,50);
        greenSlider.setMinorTickSpacing(1);
        greenSlider.setMajorTickSpacing(50);
        greenSlider.setPaintLabels(true);
        greenSlider.addChangeListener(new ChangeListener()
        {
            @Override 
            public void stateChanged(ChangeEvent e)
            {
                greenTextValue.setText(""+greenSlider.getValue());
                panel.repaint();
            }
        });
        
        greenTextValue = new JTextField();
        greenTextValue.setBounds(110,80,30,20);    
        greenTextValue.setEnabled(false);
        greenTextValue.setText(""+greenSlider.getValue());
        
        blueLabel = new JLabel();
        blueLabel.setBounds(10,140,110,20);
        blueLabel.setText("(%) of  Blue:");
        blueLabel.setFont(new Font(Font.SERIF,Font.BOLD,15));
        
        blueSlider = new JSlider();
        blueSlider.setBounds(145,130,200,50);
        blueSlider.setMinorTickSpacing(1);
        blueSlider.setMajorTickSpacing(50);
        blueSlider.setPaintLabels(true);
        blueSlider.addChangeListener(new ChangeListener()
        {
            @Override 
            public void stateChanged(ChangeEvent e)
            {
                blueTextValue.setText(""+blueSlider.getValue());
                panel.repaint();
            }
        });
        
        blueTextValue = new JTextField();
        blueTextValue.setBounds(110,140,30,20);    
        blueTextValue.setEnabled(false);
        blueTextValue.setText(""+blueSlider.getValue());
        
        confirmButton = new JButton();
        confirmButton.setBounds(200,200,120,30);
        confirmButton.setText("Process");
        panel = new JPanel()
        {
            @Override
            public void paintComponent(Graphics g)
            {
                g.setColor(Color.WHITE);
                g.fillRect(0,0, 200, 200);
                BufferedImage originalImage = Window.canvas.getOriginalSizeImage();
                Image scaledImage;
                
                scaledImage = originalImage.getScaledInstance(140,140,Image.SCALE_SMOOTH);
                
                BufferedImage GrayscaleImage = Tools.GrayScale(imageToBufferedImage(scaledImage), (double)redSlider.getValue()/100, (double)greenSlider.getValue()/100, (double)blueSlider.getValue()/100);
               
                g.drawImage(GrayscaleImage, 5, 5, null);
            }
        };
       
        panel.setBounds(360,10,150,150);
        
        this.add(panel);
        
        this.setSize(540,300);
        this.setLayout(null);
        this.add(redSlider);
        this.add(greenSlider);
        this.add(blueSlider);
        this.add(confirmButton);
        this.add(panel);
        this.add(redLabel);
        this.add(greenLabel);
        this.add(blueLabel);
        this.add(redTextValue);
        this.add(greenTextValue);
        this.add(blueTextValue);
        
        confirmButton.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                SetValues();
                setOK(true);
                setVisible(false);
            }
        });
    }
    
     /**
     * Auxiliary method for converting format Image to BufferedImage
     * @param im Input image
     * @return BufferedImage converted from the image
     */
    public static BufferedImage imageToBufferedImage(Image im) {
        BufferedImage bi = new BufferedImage
           (im.getWidth(null),im.getHeight(null),BufferedImage.TYPE_INT_RGB);
        Graphics bg = bi.getGraphics();
        bg.drawImage(im, 0, 0, null);
        bg.dispose();
        return bi;
    }
    
    /**
     * Method for setting private variables
     */
    private void SetValues()
    {
        this.redValue = redSlider.getValue();
        this.greenValue = greenSlider.getValue();
        this.blueValue = blueSlider.getValue();
    }
    
    /**
     * Method for getting private variables
     * @return array of spinner values
     */
    public double[] getValues()
    {
        return new double[] {(double)this.redValue/100, (double)this.greenValue/100, (double)this.blueValue/100};
    }
    
    /**
     * Text Field displaying the correspondent value of the slider
     */
    private JTextField redTextValue;
     /**
     * Text Field displaying the correspondent value of the slider
     */
    private JTextField greenTextValue;
    /**
     * Text Field displaying the correspondent value of the slider
     */
    private JTextField blueTextValue;
    /**
     * Red label
     */
    private JLabel redLabel;
    /**
     * Green label
     */
    private JLabel greenLabel;
    /**
     * Blue label
     */
    private JLabel blueLabel;
     /**
     * Red slider for setting the share of red color on the grayscale output
     */
    private JSlider redSlider;
        /**
     * Green slider for setting the share of green color on the grayscale output
     */
    private JSlider greenSlider;
    /**
     * Blue slider for setting the share of blue color on the grayscale output
     */
    private JSlider blueSlider;
    /**
     * Displayed confirm button for confirmation and closing the window.
     */
    private JButton confirmButton;
    /**
     * Panel for displaying the miniature of the procedure applied on the original image
     */
    private JPanel panel;
    
}
