package pogr;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
/**
 * Class representing the dialog frame for selecting the properties for Highlighting Edges
 * @author KarelSimanek
 */
public class HighlightEdgesSelector extends JDialog{
    
    /**
     * private value representing the selected value of parametr alpha
     */
    private static double value;
     /**
     * Private variable representing the successful confirmation of the dialog
     */
    private static boolean OK;
    
    /**
     * Auxiliary method for setting the confirmation to OK
     * @param value boolean operand
     */
    private void setOK(boolean value)
    {
        HighlightEdgesSelector.OK = value;
    }
    /**
     * Auxiliary method for determining the OK value
     * @return boolean value of variable OK
     */
    public boolean isOK()
    {
        return HighlightEdgesSelector.OK;
    }
    /**
     * Constructor for the custom dialog
     * @param parent inherited constructor operand
     */
    public HighlightEdgesSelector(JFrame parent)
    {
        super(parent,"HightLight edge selector");
        
        this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        this.setModal(true);
        
        this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        
        addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
            public void windowClosing(java.awt.event.WindowEvent evt) {
                setOK(false);
                setVisible(false);
            }
        });
        
        spinner = new JSpinner(new SpinnerNumberModel(0.5,0,10,0.1));
        spinner.setBounds(10,10,150,20);
        spinner.addChangeListener(new ChangeListener()
        {
            @Override 
            public void stateChanged(ChangeEvent e)
            {
                SetValues();
                panel.repaint();
            }
        });
        SetValues();
        
       
        confirmButton = new JButton();
        confirmButton.setBounds(10,200,120,30);
        confirmButton.setText("Process");
        panel = new JPanel()
        {
            @Override
            public void paintComponent(Graphics g)
            {
                BufferedImage sample;
                g.setColor(Color.WHITE);
                g.fillRect(0,0, 200, 200);
                BufferedImage originalImage = Window.canvas.getOriginalSizeImage();
                Image scaledImage;
                
                scaledImage = originalImage.getScaledInstance(140,140,Image.SCALE_SMOOTH);
                
                sample = Tools.HighlightEdges(imageToBufferedImage(scaledImage), getValues());
                g.drawImage(sample, 5, 5, null);
            }
        };
       
        panel.setBounds(10,40,150,150);
        
        this.add(panel);
        
        this.setSize(200,300);
        this.setLayout(null);
        this.add(confirmButton);
        this.add(panel);
        this.add(spinner);
        
        confirmButton.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                SetValues();
                setOK(true);
                setVisible(false);
            }
        });
    }
    /**
     * Auxiliary method for converting format Image to BufferedImage
     * @param im Input image
     * @return BufferedImage converted from the image
     */
    public static BufferedImage imageToBufferedImage(Image im) {
        BufferedImage bi = new BufferedImage
           (im.getWidth(null),im.getHeight(null),BufferedImage.TYPE_INT_RGB);
        Graphics bg = bi.getGraphics();
        bg.drawImage(im, 0, 0, null);
        bg.dispose();
        return bi;
    }
    
    /**
     * Method for setting private variables
     */
    private void SetValues()
    {
        HighlightEdgesSelector.value = (double)spinner.getValue();
    }
    
    /**
     * Method for getting private variables
     * @return array of spinner values
     */
    public double getValues()
    {
        return HighlightEdgesSelector.value;
    }
    /**
     * Displazed spinner for selecting the alpha parameter
     */
    private JSpinner spinner;
    /**
     * Displayed confirm button for confirmation and closing the window.
     */
    private JButton confirmButton;
    /**
     * Panel for displaying the miniature of the procedure applied on the original image
     */
    private JPanel panel;
    
}
