package pogr;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.Stack;
import javax.swing.JPanel;

/**
 * Class for displaying the main image
 * @author KarelSimanek
 */
public class MyCanvas extends JPanel{
    
    /**
     * DisplayedImage
     */
    private BufferedImage image;
    /**
     * Saved originalImage (image saved before zooming, etc..)
     */
    private BufferedImage originalSizeImage;
    /**
     * current zoom rate
     */
    private double zoom;
    /**
     * Stack for keeping next images
     */
    private Stack redoStack;
    /**
     * Stack for keeping previous images
     */
    private Stack undoStack;
        
    /**
     * Overridden method for painting on the panel
     * @param g Graphics parameter
     */
    @Override
    protected void paintComponent(Graphics g)
    {
        super.paintComponents(g);
        g.drawImage(image, 0, 0, null);
    }
    
    /**
     * getter for getting the current image
     */
    public BufferedImage getImage()
    {
        return this.image;
    }
    
    /**
     * setter for setting the current image
     * @param image image operand
     */
    public void setImage(BufferedImage image)
    {
        this.image = image;
    }
    
    /**
     * Method for saving the current image to undoStack
     * @param image image operand
     */
    public void saveAsPrevious(BufferedImage image)
    {
        if(undoStack == null)
            undoStack = new Stack();
        
        if(redoStack == null)
            redoStack = new Stack();
        
        undoStack.push(image);
        if (!isLastImage()) emptyRedoStack();
    }
    
    /**
     * Method for getting the previous image from the undoStack
     * @return Previous BufferedImage
     */
    public BufferedImage getPrevious()
    {
        if(undoStack == null)
            undoStack = new Stack();
        
        if(redoStack == null)
            redoStack = new Stack();
        
        if(!undoStack.isEmpty())
        {
            redoStack.push(this.originalSizeImage);
            return (BufferedImage)undoStack.pop();
        }
        else {
            return null;
        }
    }
    
    /**
     * is no next to redo?
     * @return boolean output
     */
    public boolean isLastImage()
    {
        return (redoStack.isEmpty());
    }
    
    /**
     * Method that empty the redoStack
     */
    public void emptyRedoStack()
    {
        if (redoStack != null)
            redoStack.removeAllElements();
    }
    
    /**
     * Method that empty the undoStack
     */
    public void emptyUndoStack()
    {
        if (undoStack != null)
            undoStack.removeAllElements();
    }
    
    /**
     * Get the next image from the redoStack
     * @return Next Image from the redoStack
     */
    public BufferedImage getNext()
    {
       if(undoStack == null)
            undoStack = new Stack();
        
        if(redoStack == null)
            redoStack = new Stack();
        
       if(!redoStack.isEmpty())
       {
           undoStack.push(this.originalSizeImage);
           return (BufferedImage)redoStack.pop();
       }
       else {
            return null;
       }
    }
    
    /**
     * getter for the originalSizeImage
     * @return BufferedImage in the original size
     */
    public BufferedImage getOriginalSizeImage()
    {
        return this.originalSizeImage;
    }
    
    /**
     * setter for the originalSizeImage
     * @param image BufferedImage in the current size
     */
    public void setOriginalSizeImage(BufferedImage image)
    {
        this.originalSizeImage = image;
    }
    /**
     * Setter for the zoom size
     * @param value double operand
     */
    public void setZoomValue(double value)
    {
        this.zoom = value;
    }
    /**
     * getter for the zoom size
     * @return double output
     */
    public double getZoomValue()
    {
        return this.zoom;
    }
}
