package pogr;

import java.awt.Color;
import java.awt.image.BufferedImage;
/**
 * Class with different tools
 * @author KarelSimanek
 */
public class Tools {
    
    /**
     * Rotation to the right by 90 degrees
     * @param image BufferedImage operand
     * @return BufferedImage output
     */
    public static BufferedImage rotateRight(BufferedImage image)
    {
        int originalWidth = image.getWidth();
        int originalHeight = image.getHeight();
        BufferedImage img = new BufferedImage(originalHeight,originalWidth,image.getType());
        
        for (int x = 0; x < originalWidth; x++) {
            for (int y = 0; y < originalHeight; y++) { 
                img.setRGB(originalHeight-y-1,x,image.getRGB(x, y));
              }
        }
        return img;
    }
    
    /**
     * Rotation to the left by 90 degrees
     * @param image BufferedImage operand
     * @return BufferedImage output
     */
    public static BufferedImage rotateLeft(BufferedImage image)
    {
        int originalWidth = image.getWidth();
        int originalHeight = image.getHeight();
        BufferedImage img = new BufferedImage(originalHeight,originalWidth,image.getType());
        
        for (int x = 0; x < originalWidth; x++) {
            for (int y = 0; y < originalHeight; y++) { 
                img.setRGB(y,originalWidth-x-1,image.getRGB(x, y));
              }
        }
        return img;
    }
    
    /**
     * Method for making Grayscale image
     * @param image input image 
     * @param pRed share of Red
     * @param pGreen share of Green
     * @param pBlue share of Blue
     * @return Grayscale image output
     */
    public static BufferedImage GrayScale(BufferedImage image, double pRed, double pGreen, double pBlue)
    {
        double soucet;
        if ((soucet = pRed+pGreen+pBlue) > 1) {
            pRed /= soucet;
            pGreen /= soucet;
            pBlue /= soucet;
        }
        else if (soucet < 1)
        {
                pRed *= soucet;
                pGreen *= soucet;
                pGreen *=soucet;
        }
        
        BufferedImage newImage = new BufferedImage(image.getWidth(), image.getHeight(), image.getType());
        for (int y = 0; y < image.getHeight(); y++) {
            for (int x = 0; x < image.getWidth(); x++)
            {
                Color pixelColor = new Color(image.getRGB(x, y));
                int GrayScaleColor = (int)(pixelColor.getGreen()*pGreen+pixelColor.getBlue()*pBlue+pixelColor.getRed()*pRed);
                Color GrayScaleColorRGB = new Color(GrayScaleColor,GrayScaleColor,GrayScaleColor);
                newImage.setRGB(x, y, GrayScaleColorRGB.getRGB());
            }
        }
        return newImage;        
    }
    
    /**
     * Bilinear resize
     * @param img input image   
     * @param scale scale value
     * @return new resized image
     */
    public static BufferedImage BilinearResize(BufferedImage img, double scale) {
        int newWidth = (int)(img.getWidth()*scale);
        int newHeight = (int)(img.getHeight()*scale);
        int oldWidth = img.getWidth();
        int oldHeight = img.getHeight();
        BufferedImage newImg = new BufferedImage(newWidth,newHeight,img.getType());
        int x, y, index ;
        Color a,b,c,d;
        double x_diff, y_diff;
        int blue, red, green;
        for (int i=0; i < newHeight; i++) {
            for (int j=0;j< newWidth; j++) {
                
                x = (int)(j/scale) ;
                y = (int)(i/scale) ;
                x_diff = (j/scale) - x ;
                y_diff = (i/scale) - y ;
                
                a = new Color(img.getRGB(x, y));
                if(x+1>oldWidth-1) {
                    b = new Color(0,0,0);
                }
                else {
                    b = new Color(img.getRGB(x+1,y));
                }
                if(y+1>oldHeight-1) {
                    c = new Color(0,0,0);
                }
                else {
                    c = new Color(img.getRGB(x,y+1));
                }
                if(x+1>oldWidth-1 || y+1>oldHeight-1) {
                    d = new Color(0,0,0);
                }
                else {
                    d = new Color(img.getRGB(x+1,y+1));
                }
                

                blue = (int)(a.getBlue()*(1-x_diff)*(1-y_diff) + b.getBlue()*(x_diff)*(1-y_diff) + c.getBlue()*(y_diff)*(1-x_diff) + d.getBlue()*(x_diff*y_diff));
                green = (int)(a.getGreen()*(1-x_diff)*(1-y_diff) + b.getGreen()*(x_diff)*(1-y_diff) + c.getGreen()*(y_diff)*(1-x_diff) + d.getGreen()*(x_diff*y_diff));
                red = (int)(a.getRed()*(1-x_diff)*(1-y_diff) + b.getRed()*(x_diff)*(1-y_diff) + c.getRed()*(y_diff)*(1-x_diff) + d.getRed()*(x_diff*y_diff));

                newImg.setRGB(j, i, new Color(red,green,blue).getRGB());
            }
        }
        Window.canvas.setZoomValue(scale);
        return newImg;
    }
    
    /**
     * Method for highlighting edges
     * @param image input BufferedImage
     * @param alpha parameter alpha
     * @return new image with Highlighted Edges
     */
    public static BufferedImage HighlightEdges(BufferedImage image, double alpha)
    {
        int w = image.getWidth();
        int h = image.getHeight();
        int RGB;
        int avgRed;
        int avgGreen;
        int avgBlue;
        BufferedImage newImg = new BufferedImage(w,h,image.getType());
        int Blue,Red,Green;
        
        for (int i = 0; i < w; i++)
        {
            for (int j = 0; j < h; j++)
            {
                int count = 0;
                avgRed = avgGreen = avgBlue = 0;

                for(int jj=max(j-1,0); jj<=min(j+1,h-1) ; jj++)
                        for(int ii=max(i-1,0); ii<=min(i+1,w-1) ; ii++) {
                                RGB = image.getRGB(ii,jj);
                                avgRed += (RGB >> 16) & 0xFF;
                                avgGreen += (RGB >> 8) & 0xFF;
                                avgBlue += (RGB >> 0) & 0xFF;
                                count++;
                        }

                avgRed /= count;
                avgGreen /= count;
                avgBlue /= count;
               
                Blue = (int)((new Color(image.getRGB(i, j)).getBlue()-alpha*avgBlue)/(1.-alpha));
                Red = (int)((new Color(image.getRGB(i, j)).getRed()-alpha*avgRed)/(1.-alpha));
                Green = (int)((new Color(image.getRGB(i, j)).getGreen()-alpha*avgGreen)/(1.-alpha));
                
                Blue = (Blue>=0 && Blue<=255) ? Blue : (Blue>255) ? 255 : 0;
                Red = (Red>=0 && Red<=255) ? Red : (Red>255) ? 255 : 0;
                Green = (Green>=0 && Green<=255) ? Green : (Green>255) ? 255 : 0;
                
                newImg.setRGB(i, j, new Color(Red,Green,Blue).getRGB());
            }
        }
        return newImg;
    }

    private static int min(int a, int b) {
        return (a>b)? b : a;
    }

    private static int max(int a, int b) {
        return (a>b)? a : b;
    }

}
