package pogr;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import javax.swing.BoundedRangeModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Class representing the dialog frame for selecting the properties for Tresholding
 * @author KarelSimanek
 */
public class TresholdSelector extends JDialog{
    
    /**
     * Slider Listener
     */
    private ChangeListener sliderListener = new ChangeListener()
    {
            @Override 
            public void stateChanged(ChangeEvent e)
            {
                blueTextValue.setText(""+blueSlider.getValue());
                greenTextValue.setText(""+greenSlider.getValue());
                redTextValue.setText(""+redSlider.getValue());
                
                panel.repaint();
            }
    };
    
    /**
     * red slider value
     */
    private int redValue;
    /**
     * green slider value
     */
    private int greenValue;
    /**
     * blue slider value
     */
    private int blueValue;
    
    /**
     * Private variable representing the successful confirmation of the dialog
     */
    private static boolean OK;
    
    /**
     * Auxiliary method for setting the confirmation to OK
     * @param value boolean operand
     */
    private void setOK(boolean value)
    {
        TresholdSelector.OK = value;
    }
    /**
     * Auxiliary method for determining the OK value
     * @return boolean value of variable OK
     */
    public boolean isOK()
    {
        return TresholdSelector.OK;
    }
    
    /**
     * Constructor for the custom dialog
     * @param parent inherited constructor operand
     */
    public TresholdSelector(JFrame parent)
    {
        super(parent,"Treshold value selector");
        
        this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        this.setModal(true);
        
        this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        
        addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
            public void windowClosing(java.awt.event.WindowEvent evt) {
                setOK(false);
                setVisible(false);
            }
        });
        
        models = new BoundedRangeModel[3];
        redLabel = new JLabel();
        redLabel.setBounds(10,20,110,20);
        redLabel.setText("(%) of  Red:");
        redLabel.setFont(new Font(Font.SERIF,Font.BOLD,15));
        
        redSlider = new JSlider(0,255);
        models[0] = redSlider.getModel();
        redSlider.setBounds(145,10,200,50);
        redSlider.setMinorTickSpacing(1);
        redSlider.setMajorTickSpacing(50);
        redSlider.setPaintLabels(true);
        redSlider.addChangeListener(sliderListener);
        
        redTextValue = new JTextField();
        redTextValue.setBounds(110,20,30,20);    
        redTextValue.setEnabled(false);
        redTextValue.setText(""+redSlider.getValue());
        
        greenLabel = new JLabel();
        greenLabel.setBounds(10,80,110,20);
        greenLabel.setText("(%) of  Green:");
        greenLabel.setFont(new Font(Font.SERIF,Font.BOLD,15));
        
        greenSlider = new JSlider(0,255);
        models[1] = greenSlider.getModel();
        greenSlider.setBounds(145,70,200,50);
        greenSlider.setMinorTickSpacing(1);
        greenSlider.setMajorTickSpacing(50);
        greenSlider.setPaintLabels(true);
        greenSlider.addChangeListener(sliderListener);
        
        greenTextValue = new JTextField();
        greenTextValue.setBounds(110,80,30,20);    
        greenTextValue.setEnabled(false);
        greenTextValue.setText(""+greenSlider.getValue());
        
        blueLabel = new JLabel();
        blueLabel.setBounds(10,140,110,20);
        blueLabel.setText("(%) of  Blue:");
        blueLabel.setFont(new Font(Font.SERIF,Font.BOLD,15));
        
        blueSlider = new JSlider(0,255);
        models[2] = blueSlider.getModel();
        blueSlider.setBounds(145,130,200,50);
        blueSlider.setMinorTickSpacing(1);
        blueSlider.setMajorTickSpacing(50);
        blueSlider.setPaintLabels(true);
        blueSlider.addChangeListener(sliderListener);
        
        blueTextValue = new JTextField();
        blueTextValue.setBounds(110,140,30,20);    
        blueTextValue.setEnabled(false);
        blueTextValue.setText(""+blueSlider.getValue());
        
        confirmButton = new JButton();
        confirmButton.setBounds(200,200,120,30);
        confirmButton.setText("Process");
        panel = new JPanel()
        {
            @Override
            public void paintComponent(Graphics g)
            {
                g.setColor(Color.WHITE);
                g.fillRect(0,0, 200, 200);
                BufferedImage originalImage = Window.canvas.getOriginalSizeImage();
                Image scaledImage;
                
                scaledImage = originalImage.getScaledInstance(140,140,Image.SCALE_SMOOTH);
                
                BufferedImage GrayscaleImage = Dither.Tresholding(imageToBufferedImage(scaledImage), redSlider.getValue(), greenSlider.getValue(), blueSlider.getValue());
               
                g.drawImage(GrayscaleImage, 5, 5, null);
            }
        };
       
        panel.setBounds(360,10,150,150);
        
        checkBox = new JCheckBox();
        checkBox.setBounds(360,160,30,30);
        checkBox.addActionListener(new java.awt.event.ActionListener(){
            public void actionPerformed(java.awt.event.ActionEvent ae){
                if(checkBox.isSelected())
                {
                    blueSlider.setModel(greenSlider.getModel());
                    redSlider.setModel(greenSlider.getModel());
                }
                else
                {
                    blueSlider.setModel(models[2]);
                    redSlider.setModel(models[0]);
                
                }
            }
        });
        
        lockText = new JLabel();
        lockText.setBounds(390,160,100,30);
        lockText.setText("Lock sliders");
        
        this.add(panel);

        this.setSize(540,300);
        this.setLayout(null);
        this.add(redSlider);
        this.add(greenSlider);
        this.add(blueSlider);
        this.add(confirmButton);
        this.add(panel);
        this.add(redLabel);
        this.add(greenLabel);
        this.add(blueLabel);
        this.add(redTextValue);
        this.add(greenTextValue);
        this.add(blueTextValue);
        this.add(checkBox);
        this.add(lockText);
        
        confirmButton.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                SetValues();
                setOK(true);
                setVisible(false);
            }
        });
    }
    
    /**
     * Auxiliary method for converting format Image to BufferedImage
     * @param im Input image
     * @return BufferedImage converted from the image
     */
    public static BufferedImage imageToBufferedImage(Image im) {
        BufferedImage bi = new BufferedImage
           (im.getWidth(null),im.getHeight(null),BufferedImage.TYPE_INT_RGB);
        Graphics bg = bi.getGraphics();
        bg.drawImage(im, 0, 0, null);
        bg.dispose();
        return bi;
    }
    
     /**
     * Method for setting private variables
     */
    private void SetValues()
    {
        this.redValue = redSlider.getValue();
        this.greenValue = greenSlider.getValue();
        this.blueValue = blueSlider.getValue();
    }
    /**
     * Method for getting private variables
     * @return array of spinner values
     */
    public int[] getValues()
    {
        return new int[] {this.redValue, this.greenValue, this.blueValue};
    }
    
    /**
     * array of slider models
     */
    private BoundedRangeModel[] models;
    private JTextField redTextValue;
    private JTextField greenTextValue;
    private JTextField blueTextValue;
    private JLabel redLabel;
    private JLabel greenLabel;
    private JLabel blueLabel;
    private JSlider redSlider;
    private JSlider greenSlider;
    private JSlider blueSlider;
    private JButton confirmButton;
    private JPanel panel;
    private JCheckBox checkBox;
    private JLabel lockText;
    
    
}
